import 'dart:async';
import 'dart:io';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:ocg_story/src/models/basic.dart';
import 'package:ocg_story/src/widgets/camera_button.dart';
import 'package:ocg_story/src/widgets/switch_icon.dart';
import 'package:ocg_story/src/widgets/thumbnail_widget.dart';
import 'package:ocg_story/src/widgets/video_button.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:video_thumbnail/video_thumbnail.dart' as thumbnail;

class CameraScreen extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _CameraScreenState();
}

class _CameraScreenState extends State<CameraScreen>
    with WidgetsBindingObserver, TickerProviderStateMixin<CameraScreen> {
  CameraController controller;
  TabController tabController;
  String videoPath;
  VoidCallback videoPlayerListener;
  String imagePath;
  List<CameraDescription> cameras;
  bool isVideoSelected;
  bool isVideoRecording;

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  void initState() {
    super.initState();
    isVideoSelected = false;
    isVideoRecording = false;
    SystemChrome.setEnabledSystemUIOverlays([]);
    WidgetsBinding.instance.addObserver(this);
    tabController = TabController(length: 2, vsync: this);
    getCameras();
  }

  @override
  void dispose() {
    SystemChrome.setEnabledSystemUIOverlays(
        [SystemUiOverlay.top, SystemUiOverlay.bottom]);
    WidgetsBinding.instance.removeObserver(this);
    controller.dispose();
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.inactive) {
      controller?.dispose();
    } else if (state == AppLifecycleState.resumed) {
      if (controller != null) {
        onNewCameraSelected(controller.description);
      }
    }
  }

  void onNewCameraSelected(CameraDescription cameraDescription) async {
    if (controller != null) {
      await controller.dispose();
    }
    controller = CameraController(cameraDescription, ResolutionPreset.medium);

    // If the controller is updated then update the UI.
    controller.addListener(() {
      if (mounted) setState(() {});
      if (controller.value.hasError) {
        showInSnackBar('Camera error ${controller.value.errorDescription}');
      }
    });

    try {
      await controller.initialize();
    } on CameraException catch (e) {
      _showCameraException(e);
    }

    if (mounted) {
      setState(() {});
    }
  }

  Future getCameras() async {
    Map<Permission, PermissionStatus> result = await [
      Permission.camera,
      Permission.microphone,
      Permission.mediaLibrary,
      Permission.storage,
    ].request();

    bool isPermitted = result.values
        .map((e) => e == PermissionStatus.granted)
        .reduce((value, element) => value && element);
    if (!isPermitted) {
      return;
    }
    cameras = await availableCameras();
    if (cameras.length < 1) {
      return;
    }
    controller = CameraController(cameras[0], ResolutionPreset.high);
    controller.initialize().then((_) {
      if (!mounted) {
        return;
      }
      setState(() {});
    });
  }

  String timestamp() => DateTime.now().millisecondsSinceEpoch.toString();
  Future<String> takePicture({bool video = false}) async {
    if (!controller.value.isInitialized) {
      showInSnackBar('Error: select a camera first.');
      return null;
    }
    Directory extDir;
    extDir = await getTemporaryDirectory();

    final String dirPath = '${extDir.path}/Pictures/flutter_test';
    await Directory(dirPath).create(recursive: true);
    final String filePath = '$dirPath/${timestamp()}.jpg';
    if (controller.value.isTakingPicture) {
      return null;
    }
    try {
      await controller.takePicture(filePath);
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
    if (!video) {
      imagePath = filePath;
      setState(() {});
      sendImage();
    } else {
      imagePath = filePath;
    }
    return filePath;
  }

  Future<void> sendImage() async {
    Navigator.of(context).pop(ImageData.named(imagePath: imagePath));
  }

  void _showCameraException(CameraException e) {
    print(e.code + e.description);
    showInSnackBar('Error: ${e.code}\n${e.description}');
  }

  bool isPermitted = false;
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _scaffoldKey,
      body: Center(
        child: Stack(
          children: <Widget>[_cameraPreviewWidget(), getOptionsWidget()],
        ),
      ),
    );
  }

  Widget getOptionsWidget() {
    return Scaffold(
      backgroundColor: Colors.transparent,
      appBar: AppBar(
        backgroundColor: Colors.transparent,
        elevation: 0.0,
        iconTheme: IconThemeData(color: Colors.white),
        actionsIconTheme: IconThemeData(color: Colors.white),
        leading: CloseButton(),
        // actions: <Widget>[
        //   IconButton(
        //       icon: Icon(Icons.flash_off),
        //       onPressed: () {
        //         showInSnackBar("Flash not implemented yet");
        //       }),
        //   IconButton(
        //       icon: Icon(Icons.brightness_2),
        //       onPressed: () {
        //         showInSnackBar("Feature not implemented");
        //       }),
        // ],
      ),
      bottomNavigationBar: getCameraButtonRow(),
    );
  }

  Widget _cameraPreviewWidget() {
    final size = MediaQuery.of(context).size;
    if (controller == null || !controller.value.isInitialized) {
      return const Text(
        'Tap a camera',
        style: TextStyle(
          color: Colors.black,
          fontSize: 24.0,
          fontWeight: FontWeight.w900,
        ),
      );
    } else {
      return Transform.scale(
        scale: controller.value.aspectRatio / size.aspectRatio,
        child: Center(
          child: AspectRatio(
            aspectRatio: controller.value.aspectRatio,
            child: CameraPreview(controller),
          ),
        ),
      );
    }
  }

  Widget _getThumbnail() {
    return ThumbnailWidget(
      imagePath: imagePath,
      size: 36.0,
      isVideo: tabController.index == 1,
      onPicked: (file, isVideo) {
        if (isVideo) {
          videoPath = file;
          sendVideo();
        } else {
          imagePath = file;
          sendImage();
        }
      },
    );
  }

  Widget _getCameraSwitch() {
    return controller != null && tabController.index == 0
        ? SwitchIcon(
            size: 24.0,
            onTap: () {
              CameraLensDirection direction =
                  controller.description.lensDirection;
              CameraLensDirection required =
                  direction == CameraLensDirection.front
                      ? CameraLensDirection.back
                      : CameraLensDirection.front;
              for (CameraDescription cameraDescription in cameras) {
                if (cameraDescription.lensDirection == required) {
                  onNewCameraSelected(cameraDescription);
                  return;
                }
              }
            },
          )
        : Container(
            width: 0,
            height: 0,
          );
  }

  Widget getCameraButtonRow() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        isVideoSelected
            ? VideoButton(
                takeVideo: takeVideo,
                stopVideo: stopVideo,
                isVideoRecording: isVideoRecording,
              )
            : CameraButton(
                takePicture: takePicture,
              ),
        Padding(
          padding: const EdgeInsets.symmetric(horizontal: 16.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: <Widget>[
              _getThumbnail(),
              Expanded(
                child: Container(
                  height: 50.0,
                  alignment: Alignment.center,
                  padding: const EdgeInsets.symmetric(horizontal: 48.0),
                  child: TabBar(
                    isScrollable: false,
                    tabs: [
                      Tab(
                        text: "Camera",
                      ),
                      Tab(
                        text: "Video",
                      ),
                    ],
                    indicatorColor: Colors.white,
                    indicatorWeight: 3.0,
                    indicatorSize: TabBarIndicatorSize.tab,
                    controller: tabController,
                    labelColor: Colors.white,
                    unselectedLabelColor: Colors.white.withAlpha(160),
                    onTap: (index) {
                      if (index == 1) {
                        isVideoSelected = true;
                      } else
                        isVideoSelected = false;
                      setState(() {
                        stopVideo();
                        isVideoRecording = false;
                      });
                    },
                  ),
                ),
              ),
              _getCameraSwitch()
            ],
          ),
        ),
      ],
    );
  }

  Future<Directory> getDirectory() async {
    if (Platform.isIOS) {
      return await getApplicationDocumentsDirectory();
    } else {
      return await getTemporaryDirectory();
    }
  }

  // Start Video Recording
  Future<String> takeVideo() async {
    setState(() {
      isVideoRecording = true;
    });
    if (!controller.value.isInitialized) {
      showInSnackBar('Error: select a camera first.');
      return null;
    }
    Directory extDir;
    extDir = await getDirectory();
    final String dirPath = '${extDir.path}/Movies/flutter_test';
    await Directory(dirPath).create(recursive: true);
    final String filePath = '$dirPath/${timestamp()}.mp4';
    await takePicture(video: true);
    if (controller.value.isRecordingVideo) {
      return null;
    }

    try {
      videoPath = filePath;
      await controller.startVideoRecording(filePath);
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
    print(filePath);

    return filePath;
  }

  Future<void> stopVideo() async {
    setState(() {
      isVideoRecording = false;
    });

    if (!controller.value.isRecordingVideo) {
      return null;
    }
    try {
      await controller.stopVideoRecording();
      if (videoPath != null) {
        sendVideo();
      }
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
    setState(() {});
  }

  Future<void> sendVideo() async {
    if (imagePath == null) {
      final uint8list = await thumbnail.VideoThumbnail.thumbnailData(
        video: videoPath,
        imageFormat: thumbnail.ImageFormat.JPEG,
        maxWidth:
            128, // specify the width of the thumbnail, let the height auto-scaled to keep the source aspect ratio
        quality: 75,
      );
      Directory extDir;
      extDir = await getDirectory();
      final String dirPath = '${extDir.path}/Movies/flutter_test';
      await Directory(dirPath).create(recursive: true);
      final String filePath = '$dirPath/${timestamp()}.jpeg';
      await File(filePath).writeAsBytes(uint8list);
      imagePath = filePath;
    }
    Navigator.of(context)
        .pop(VideoData.named(videoPath: videoPath, thumbnailPath: imagePath));
  }

  void showInSnackBar(String message) {
    _scaffoldKey.currentState.showSnackBar(SnackBar(content: Text(message)));
  }
}
